@@ -2,7 +2,7 @@
 Pyro package. Some generic init stuff to set up logging etc.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 import sys
@@ -56,7 +56,10 @@ del _configLogging
 # initialize Pyro's configuration
 from Pyro4.configuration import Configuration
 config=Configuration()
+del Configuration
 
 # import the required Pyro symbols into this package
-from Pyro4.core import URI, Proxy, Daemon, callback
+from Pyro4.core import URI, Proxy, Daemon, callback, batch, async
 from Pyro4.naming import locateNS, resolve
+from Pyro4.constants import VERSION as __version__
+
