@@ -27,18 +27,18 @@ log=logging.getLogger("Pyro.core")
 
 
 class URI(object):
-    """Pyro object URI (universal resource identifier)
-    PYRO uri format: PYRO:objectid@location
-        where location is one of:
-          hostname:port  (tcp/ip socket on given port)
-          ./p:pipename   (named pipe on localhost)
-          ./u:sockname   (unix domain socket on localhost)
+    """
+    Pyro object URI (universal resource identifier).
+    The uri format is like this: ``PYRO:objectid@location`` where location is one of:
 
-    MAGIC URI format for simple name resolution using Name server:
-      PYRONAME:objectname[@location]  (optional name server location, can also omit location port)
+    - ``hostname:port`` (tcp/ip socket on given port)
+    - ``./u:sockname`` (unix domain socket on localhost)
+
+    There is also a 'Magic format' for simple name resolution using Name server:
+      ``PYRONAME:objectname[@location]``  (optional name server location, can also omit location port)
     """
     uriRegEx=re.compile(r"(?P<protocol>PYRO[A-Z]*):(?P<object>\S+?)(@(?P<location>\S+))?$")
-    __slots__=("protocol", "object", "pipename", "sockname", "host", "port", "object")
+    __slots__=("protocol", "object", "sockname", "host", "port", "object")
 
     def __init__(self, uri):
         if isinstance(uri, URI):
@@ -47,7 +47,7 @@ class URI(object):
             return
         if not isinstance(uri, basestring):
             raise TypeError("uri parameter object is of wrong type")
-        self.pipename=self.sockname=self.host=self.port=None
+        self.sockname=self.host=self.port=None
         match=self.uriRegEx.match(uri)
         if not match:
             raise errors.PyroError("invalid uri")
@@ -67,13 +67,9 @@ class URI(object):
     def _parseLocation(self, location, defaultPort):
         if not location:
             return
-        if location.startswith("./p:"):
-            self.pipename=location[4:]
-            if (not self.pipename) or ':' in self.pipename or '/' in self.pipename:
-                raise errors.PyroError("invalid uri (location)")
-        elif location.startswith("./u:"):
+        if location.startswith("./u:"):
             self.sockname=location[4:]
-            if (not self.sockname) or ':' in self.sockname or '/' in self.sockname:
+            if (not self.sockname) or ':' in self.sockname:
                 raise errors.PyroError("invalid uri (location)")
         else:
             self.host, _, self.port=location.partition(":")
@@ -85,8 +81,8 @@ class URI(object):
                 raise errors.PyroError("invalid uri (port)")
 
     @staticmethod
-    def isPipeOrUnixsockLocation(location):
-        return location.startswith("./p:") or location.startswith("./u:")
+    def isUnixsockLocation(location):
+        return location.startswith("./u:")
 
     @property
     def location(self):
@@ -94,8 +90,6 @@ class URI(object):
             return "%s:%d" % (self.host, self.port)
         elif self.sockname:
             return "./u:"+self.sockname
-        elif self.pipename:
-            return "./p:"+self.pipename
         else:
             return None
 
@@ -110,17 +104,17 @@ class URI(object):
         return self.asString()
 
     def __eq__(self, other):
-        return (self.protocol, self.object, self.pipename, self.sockname, self.host, self.port) \
-                == (other.protocol, other.object, other.pipename, other.sockname, other.host, other.port)
+        return (self.protocol, self.object, self.sockname, self.host, self.port) \
+                == (other.protocol, other.object, other.sockname, other.host, other.port)
     __hash__=object.__hash__
     # note: getstate/setstate are not needed if we use pickle protocol 2,
     # but this way it helps pickle to make the representation smaller by omitting all attribute names.
 
     def __getstate__(self):
-        return self.protocol, self.object, self.pipename, self.sockname, self.host, self.port
+        return self.protocol, self.object, self.sockname, self.host, self.port
 
     def __setstate__(self, state):
-        self.protocol, self.object, self.pipename, self.sockname, self.host, self.port = state
+        self.protocol, self.object, self.sockname, self.host, self.port = state
 
 
 class _RemoteMethod(object):
@@ -170,7 +164,7 @@ class Proxy(object):
         if name in Proxy.__pyroAttributes:
             # allows it to be safely pickled
             raise AttributeError(name)
-        return _RemoteMethod(self.__pyroInvoke, name)
+        return _RemoteMethod(self._pyroInvoke, name)
 
     def __str__(self):
         return "<Pyro Proxy for "+str(self._pyroUri)+">"
@@ -219,7 +213,7 @@ class Proxy(object):
             self._pyroConnection.timeout=timeout
     _pyroTimeout=property(__pyroGetTimeout, __pyroSetTimeout)
 
-    def __pyroInvoke(self, methodname, vargs, kwargs, flags=0):
+    def _pyroInvoke(self, methodname, vargs, kwargs, flags=0):
         """perform the remote method call communication"""
         if not self._pyroConnection:
             # rebind here, don't do it from inside the invoke because deadlock will occur
@@ -268,50 +262,48 @@ class Proxy(object):
         """Connects this proxy to the remote Pyro daemon. Does connection handshake."""
         from Pyro4.naming import resolve  # don't import this globally because of cyclic dependancy
         uri=resolve(self._pyroUri)
-        if uri.host and uri.port:
-            # socket connection
-            conn=None
-            log.debug("connecting to %s", uri)
-            try:
-                with self.__pyroLock:
-                    sock=socketutil.createSocket(connect=(uri.host, uri.port), timeout=self.__pyroTimeout)
-                    conn=socketutil.SocketConnection(sock, uri.object)
-                    # Do handshake. For now, no need to send anything.
-                    msgType, flags, seq, data = MessageFactory.getMessage(conn, None)
-                    # any trailing data (dataLen>0) is an error message, if any
-            except Exception:
-                x=sys.exc_info()[1]
-                if conn:
-                    conn.close()
-                err="cannot connect: %s" % x
-                log.error(err)
-                if isinstance(x, errors.CommunicationError):
-                    raise
-                else:
-                    raise errors.CommunicationError(err)
+        # socket connection (normal or unix domain socket)
+        conn=None
+        log.debug("connecting to %s", uri)
+        connect_location=uri.sockname if uri.sockname else (uri.host, uri.port)
+        try:
+            with self.__pyroLock:
+                sock=socketutil.createSocket(connect=connect_location, timeout=self.__pyroTimeout)
+                conn=socketutil.SocketConnection(sock, uri.object)
+                # Do handshake. For now, no need to send anything.
+                msgType, flags, seq, data = MessageFactory.getMessage(conn, None)
+                # any trailing data (dataLen>0) is an error message, if any
+        except Exception:
+            x=sys.exc_info()[1]
+            if conn:
+                conn.close()
+            err="cannot connect: %s" % x
+            log.error(err)
+            if isinstance(x, errors.CommunicationError):
+                raise
             else:
-                if msgType==MessageFactory.MSG_CONNECTFAIL:
-                    error="connection rejected"
-                    if data:
-                        if sys.version_info>=(3,0):
-                            data=str(data,"utf-8")
-                        error+=", reason: "+data
-                    conn.close()
-                    log.error(error)
-                    raise errors.CommunicationError(error)
-                elif msgType==MessageFactory.MSG_CONNECTOK:
-                    self._pyroConnection=conn
-                    if replaceUri:
-                        log.debug("replacing uri with bound one")
-                        self._pyroUri=uri
-                    log.debug("connected to %s", self._pyroUri)
-                else:
-                    conn.close()
-                    err="connect: invalid msg type %d received" % msgType
-                    log.error(err)
-                    raise errors.ProtocolError(err)
+                raise errors.CommunicationError(err)
         else:
-            raise NotImplementedError("non-socket uri connections not yet implemented")
+            if msgType==MessageFactory.MSG_CONNECTFAIL:
+                error="connection rejected"
+                if data:
+                    if sys.version_info>=(3,0):
+                        data=str(data,"utf-8")
+                    error+=", reason: "+data
+                conn.close()
+                log.error(error)
+                raise errors.CommunicationError(error)
+            elif msgType==MessageFactory.MSG_CONNECTOK:
+                self._pyroConnection=conn
+                if replaceUri:
+                    log.debug("replacing uri with bound one")
+                    self._pyroUri=uri
+                log.debug("connected to %s", self._pyroUri)
+            else:
+                conn.close()
+                err="connect: invalid msg type %d received" % msgType
+                log.error(err)
+                raise errors.ProtocolError(err)
 
     def _pyroReconnect(self, tries=100000000):
         self._pyroRelease()
@@ -328,16 +320,17 @@ class Proxy(object):
         raise errors.ConnectionClosedError(msg)
 
     def _pyroBatch(self):
-        return _BatchProxy(self.__pyroInvokeBatch)
+        return _BatchProxyAdapter(self)
 
-    def __pyroInvokeBatch(self, calls, oneway=False):
+    def _pyroAsync(self):
+        return _AsyncProxyAdapter(self)
+
+    def _pyroInvokeBatch(self, calls, oneway=False):
         flags=MessageFactory.FLAGS_BATCH
         if oneway:
             flags|=MessageFactory.FLAGS_ONEWAY
-        return self.__pyroInvoke("<batch>", calls, None, flags)
+        return self._pyroInvoke("<batch>", calls, None, flags)
 
-    def _pyroAsync(self, callback=None):
-        return _AsyncProxy(self.__pyroInvoke, callback)
 
 
 class _BatchedRemoteMethod(object):
@@ -353,19 +346,28 @@ class _BatchedRemoteMethod(object):
         self.__calls.append((self.__name, args, kwargs))
 
 
-class _BatchProxy(object):
+class _BatchProxyAdapter(object):
     """Helper class that lets you batch multiple method calls into one.
-    It is constructed with a reference to the method of the normal proxy that will
+    It is constructed with a reference to the normal proxy that will
     carry out the batched calls. Call methods on this object thatyou want to batch,
     and finally call the batch proxy itself. That call will return a generator
     for the results of every method call in the batch (in sequence)."""
-    def __init__(self, batchmethod):
-        self.__batchmethod=batchmethod
+    def __init__(self, proxy):
+        self.__proxy=proxy
         self.__calls=[]
 
     def __getattr__(self, name):
         return _BatchedRemoteMethod(self.__calls, name)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        pass
+
+    def __copy__(self):
+        return self
+
     def __resultsgenerator(self, results):
         for result in results:
             if isinstance(result, _ExceptionWrapper):
@@ -373,57 +375,55 @@ class _BatchProxy(object):
             else:
                 yield result   # it is a regular result object, yield that and continue.
 
-    def __call__(self, oneway=False, async=False, callback=None):
+    def __call__(self, oneway=False, async=False):
         if oneway and async:
-            raise ValueError("async oneway calls make no sense")
+            raise errors.PyroError("async oneway calls make no sense")
         if async:
-            return _AsyncRemoteMethod(self.__call_async, callback, "<asyncbatch>")()
+            return _AsyncRemoteMethod(self, "<asyncbatch>")()
         else:
-            results=self.__batchmethod(self.__calls, oneway)
+            results=self.__proxy._pyroInvokeBatch(self.__calls, oneway)
             self.__calls=[]   # clear for re-use
             if not oneway:
                 return self.__resultsgenerator(results)
 
-    def __call_async(self,name,args,kwargs):
+    def _pyroInvoke(self,name,args,kwargs):
         # ignore all parameters, we just need to execute the batch
-        results=self.__batchmethod(self.__calls)
+        results=self.__proxy._pyroInvokeBatch(self.__calls)
         self.__calls=[]   # clear for re-use
         return self.__resultsgenerator(results)
 
 
-class _AsyncProxy(object):
-    def __init__(self, invokemethod, callback):
-        self.__invokemethod=invokemethod
-        self.__callback=callback
+class _AsyncProxyAdapter(object):
+    def __init__(self, proxy):
+        self.__proxy=proxy
 
     def __getattr__(self, name):
-        return _AsyncRemoteMethod(self.__invokemethod, self.__callback, name)
+        return _AsyncRemoteMethod(self.__proxy, name)
 
 
 class _AsyncRemoteMethod(object):
     """async method call abstraction (call will run in a background thread)"""
-    def __init__(self, send, callback, name):
-        self.__send = send
+    def __init__(self, proxy, name):
+        self.__proxy = proxy
         self.__name = name
-        self.__callback = callback
 
     def __getattr__(self, name):
-        return _AsyncRemoteMethod(self.__send, self.__callback, "%s.%s" % (self.__name, name))
+        return _AsyncRemoteMethod(self.__proxy, "%s.%s" % (self.__name, name))
 
     def __call__(self, *args, **kwargs):
         asyncresult=_AsyncResult()
-        thread=threadutil.Thread(target=self.__asynccall, args=(asyncresult, self.__callback, args, kwargs))
+        thread=threadutil.Thread(target=self.__asynccall, args=(asyncresult, args, kwargs))
         thread.setDaemon(True)
         thread.start()
         return asyncresult
 
-    def __asynccall(self, asyncresult, callback, args, kwargs):
+    def __asynccall(self, asyncresult, args, kwargs):
         try:
-            value = self.__send(self.__name, args, kwargs)
-            if callback:
-                callback(value)  # only provide the result value through the callback
-            else:
-                asyncresult.value=value
+            # use a copy of the proxy otherwise calls would be serialized,
+            # and use contextmanager to close the proxy after we're done
+            with self.__proxy.__copy__() as proxy:
+                value = proxy._pyroInvoke(self.__name, args, kwargs)
+            asyncresult.value=value
         except:
             log.warn("exception occurred in async call, ignored")
             asyncresult.value=_ExceptionWrapper(sys.exc_info()[1])
@@ -432,6 +432,8 @@ class _AsyncRemoteMethod(object):
 class _AsyncResult(object):
     def __init__(self):
         self.__ready=threadutil.Event()
+        self.callchain=[]
+        self.valueLock=threadutil.Lock()
     def ready(self, timeout=None):
         if timeout is None:
             return self.__ready.isSet()
@@ -448,9 +450,22 @@ class _AsyncResult(object):
         else:
             return self.__value
     def set_value(self, value):
-        self.__value=value
-        self.__ready.set()
+        with self.valueLock:
+            self.__value=value
+            for call, kwargs in self.callchain:
+                self.__value=call(self.__value, **kwargs)
+            self.callchain=[]
+            self.__ready.set()
     value=property(get_value, set_value)
+    def then(self, call, **kwargs):
+        if self.__ready.isSet():
+            # value is already known, we need to process it immediately (can't use the callchain anymore)
+            self.__value=call(self.__value, **kwargs)
+        else:
+            # add the call to the callchain, it will be processed later when the result arrives
+            with self.valueLock:
+                self.callchain.append((call,kwargs))
+        return self
 
 
 class _ExceptionWrapper(object):
@@ -468,12 +483,12 @@ class _ExceptionWrapper(object):
 
 
 def batch(proxy):
-    """convenience method to get a batch proxy"""
+    """convenience method to get a batch proxy adapter"""
     return proxy._pyroBatch()
 
-def async(proxy, callback=None):
-    """convenience method to get an async proxy"""
-    return proxy._pyroAsync(callback)
+def async(proxy):
+    """convenience method to get an async proxy adapter"""
+    return proxy._pyroAsync()
 
 
 class MessageFactory(object):
@@ -574,9 +589,11 @@ class DaemonObject(object):
         self.daemon=daemon
 
     def registered(self):
+        """returns a list of all object names registered in this daemon"""
         return list(self.daemon.objectsById.keys())
 
     def ping(self):
+        """a simple do-nothing method for testing purposes"""
         pass
 
 
@@ -585,7 +602,7 @@ class Daemon(object):
     Pyro daemon. Contains server side logic and dispatches incoming remote method calls
     to the appropriate objects.
     """
-    def __init__(self, host=None, port=0):
+    def __init__(self, host=None, port=0, unixsocket=None):
         _check_hmac()  # check if hmac secret key is set
         if host is None:
             host=Pyro4.config.HOST
@@ -602,7 +619,7 @@ class Daemon(object):
                 self.transportServer=SocketServer_Select()
         else:
             raise errors.PyroError("invalid server type '%s'" % Pyro4.config.SERVERTYPE)
-        self.transportServer.init(self, host, port)
+        self.transportServer.init(self, host, port, unixsocket)
         self.locationStr=self.transportServer.locationStr
         log.debug("created daemon on %s", self.locationStr)
         self.serializer=util.Serializer()
@@ -680,7 +697,7 @@ class Daemon(object):
         self.__loopstopped.wait()
         log.info("daemon %s shut down", self.locationStr)
 
-    def handshake(self, conn):
+    def _handshake(self, conn):
         """Perform connection handshake with new clients"""
         # For now, client is not sending anything. Just respond with a CONNECT_OK.
         # We need a minimal amount of data or the socket will remain blocked
@@ -758,15 +775,15 @@ class Daemon(object):
                 conn.send(msg)
         except Exception:
             xt,xv=sys.exc_info()[0:2]
-            log.debug("Exception occurred while handling request: %s", xv)
+            log.debug("Exception occurred while handling request: %r", xv)
             if not flags & MessageFactory.FLAGS_ONEWAY:
                 # only return the error to the client if it wasn't a oneway call
                 tblines=util.formatTraceback(detailed=Pyro4.config.DETAILED_TRACEBACK)
-                self.sendExceptionResponse(conn, seq, xv, tblines)
-            if isCallback or xt in (errors.CommunicationError, errors.SecurityError):
+                self._sendExceptionResponse(conn, seq, xv, tblines)
+            if isCallback or isinstance(xv, (errors.CommunicationError, errors.SecurityError)):
                 raise       # re-raise if flagged as callback, communication or security error.
 
-    def sendExceptionResponse(self, connection, seq, exc_value, tbinfo):
+    def _sendExceptionResponse(self, connection, seq, exc_value, tbinfo):
         """send an exception back including the local traceback info"""
         exc_value._pyroTraceback=tbinfo
         if sys.platform=="cli":
@@ -899,4 +916,3 @@ def callback(object):
     """
     object._pyroCallback=True
     return object
-
