@@ -76,13 +76,17 @@ def main(args):
     parser.add_option("-n", "--host", dest="host", help="hostname of the NS")
     parser.add_option("-p", "--port", dest="port", type="int",
                       help="port of the NS (or bc-port if host isn't specified)")
+    parser.add_option("-u","--unixsocket", help="unix domain socket name of the NS")
     parser.add_option("-v", "--verbose", action="store_true", dest="verbose", help="verbose output")
     options, args = parser.parse_args(args)
     if not args or args[0] not in ("register", "remove", "removematching", "list", "listmatching", "ping"):
         parser.error("invalid or missing command")
     if options.verbose:
         print("Locating name server...")
+    if options.unixsocket:
+        options.host="./u:"+options.unixsocket
     try:
+
         nameserver=naming.locateNS(options.host, options.port)
     except errors.PyroError:
         x=sys.exc_info()[1]
