@@ -5,7 +5,7 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
 irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
-import socket, select, sys, logging
+import socket, select, sys, logging, os
 from Pyro4 import socketutil, errors
 import Pyro4
 
@@ -14,19 +14,23 @@ log=logging.getLogger("Pyro.socketserver.multiplexed")
 
 class MultiplexedSocketServerBase(object):
     """base class for multiplexed transport server for socket connections"""
-    def init(self, daemon, host, port):
+    def init(self, daemon, host, port, unixsocket=None):
         log.info("starting multiplexed socketserver")
         self.sock=None
-        self.sock=socketutil.createSocket(bind=(host, port), timeout=Pyro4.config.COMMTIMEOUT, noinherit=True)
+        bind_location=unixsocket if unixsocket else (host, port)
+        self.sock=socketutil.createSocket(bind=bind_location, timeout=Pyro4.config.COMMTIMEOUT, noinherit=True)
         self.clients=[]
         self.daemon=daemon
         sockaddr=self.sock.getsockname()
         if sockaddr[0].startswith("127."):
             if host is None or host.lower()!="localhost" and not host.startswith("127."):
                 log.warn("weird DNS setup: %s resolves to localhost (127.x.x.x)", host)
-        host=host or sockaddr[0]
-        port=port or sockaddr[1]
-        self.locationStr="%s:%d" % (host, port)
+        if unixsocket:
+            self.locationStr="./u:"+unixsocket
+        else:
+            host=host or sockaddr[0]
+            port=port or sockaddr[1]
+            self.locationStr="%s:%d" % (host, port)
 
     def __del__(self):
         if self.sock is not None:
@@ -69,7 +73,7 @@ class MultiplexedSocketServerBase(object):
             raise
         try:
             conn=socketutil.SocketConnection(csock)
-            if self.daemon.handshake(conn):
+            if self.daemon._handshake(conn):
                 return conn
         except (socket.error, errors.PyroError):
             x=sys.exc_info()[1]
@@ -80,7 +84,16 @@ class MultiplexedSocketServerBase(object):
     def close(self):
         log.debug("closing socketserver")
         if self.sock:
+            sockname=None
+            try:
+                sockname=self.sock.getsockname()
+            except socket.error:
+                pass
             self.sock.close()
+            if type(sockname) is str:
+                # it was a unix domain socket, remove it from the filesystem
+                if os.path.exists(sockname):
+                    os.remove(sockname)
         self.sock=None
         for c in self.clients:
             try:
@@ -139,8 +152,9 @@ class SocketServer_Poll(MultiplexedSocketServerBase):
                             except socket.error:
                                 pass
                             else:
-                                poll.unregister(fn)
-                                del fileno2connection[fn]
+                                if fn in fileno2connection:
+                                    poll.unregister(fn)
+                                    del fileno2connection[fn]
                                 conn.close()
         except KeyboardInterrupt:
             log.debug("stopping on break signal")
