@@ -35,10 +35,16 @@ def getMyIpAddress(hostname=None, workaround127=False):
     as being localhost (this is often the case on Linux)"""
     ip=getIpAddress(hostname)
     if ip.startswith("127.") and workaround127:
-        s=socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        s.connect(("4.2.2.2", 53))   # 'abuse' a level 3 DNS server
-        ip=s.getsockname()[0]
-        s.close()
+        ip=getInterfaceAddress("4.2.2.2")   # 'abuse' a level 3 DNS server
+    return ip
+
+
+def getInterfaceAddress(peer_ip_address):
+    """tries to find the ip address of the interface that connects to a given host"""
+    s=socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+    s.connect((peer_ip_address, 53))   # 53=dns
+    ip=s.getsockname()[0]
+    s.close()
     return ip
 
 
@@ -125,10 +131,12 @@ def receiveData(sock, size):
 
 
 def sendData(sock, data):
-    """Send some data over a socket."""
-    # Some OS-es have problems with sendall when the socket is in non-blocking mode.
-    # For instance, Mac OS X seems to be happy to throw EAGAIN errors too often.
-    # We fall back to using a regular send loop if needed.
+    """
+    Send some data over a socket.
+    Some systems have problems with ``sendall()`` when the socket is in non-blocking mode.
+    For instance, Mac OS X seems to be happy to throw EAGAIN errors too often.
+    This function falls back to using a regular send loop if needed.
+    """
     if sock.gettimeout() is None:
         # socket is in blocking mode, we can use sendall normally.
         while True:
@@ -159,14 +167,21 @@ def sendData(sock, data):
 
 
 def createSocket(bind=None, connect=None, reuseaddr=True, keepalive=True, timeout=None, noinherit=False):
-    """Create a socket. Default options are keepalives and reuseaddr."""
+    """
+    Create a socket. Default options are keepalives and reuseaddr.
+    If 'bind' or 'connect' is a string, it is assumed a unix domain socket is requested.
+    Otherwise, a normal tcp/ip socket is used.
+    """
     if timeout==0:
         timeout=None
     if bind and connect:
         raise ValueError("bind and connect cannot both be specified at the same time")
-    sock=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    family=socket.AF_INET
+    if type(bind) is str or type(connect) is str:
+        family=socket.AF_UNIX
+    sock=socket.socket(family, socket.SOCK_STREAM)
     if bind:
-        if bind[1]==0:
+        if type(bind) is tuple and bind[1]==0:
             bindOnUnusedPort(sock, bind[0])
         else:
             sock.bind(bind)
@@ -203,45 +218,35 @@ def createBroadcastSocket(bind=None, reuseaddr=True, timeout=None):
         else:
             sock.settimeout(timeout)
     if bind:
-        if bind[0]:
-            hosts=[bind[0]]
+        host,port=bind
+        host=host or ""
+        if port==0:
+            bindOnUnusedPort(sock, host)
         else:
-            hosts=["<broadcast>", "", "255.255.255.255"]
-        for host in hosts:
-            try:
-                if bind[1]==0:
-                    bindOnUnusedPort(sock, host)
-                else:
-                    sock.bind((host, bind[1]))
-                return sock
-            except socket.error:
-                continue
-        sock.close()
-        raise CommunicationError("cannot bind broadcast socket")
+            sock.bind((host,port))
     return sock
 
 
 def setReuseAddr(sock):
-    """sets the SO_REUSEADDR option on the socket."""
+    """sets the SO_REUSEADDR option on the socket, if possible."""
     try:
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     except Exception:
-        log.info("cannot set SO_REUSEADDR")
+        pass
 
 
 def setKeepalive(sock):
-    """sets the SO_KEEPALIVE option on the socket."""
+    """sets the SO_KEEPALIVE option on the socket, if possible."""
     try:
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
     except Exception:
-        log.info("cannot set SO_KEEPALIVE")
+        pass
 
-# set socket to not inherit in subprocess
 try:
     import fcntl
 
     def setNoInherit(sock):
-        # Mark the given socket fd as non-inheritable (posix)
+        """Mark the given socket fd as non-inheritable to child processes"""
         fd = sock.fileno()
         flags = fcntl.fcntl(fd, fcntl.F_GETFD)
         fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
@@ -252,18 +257,19 @@ except ImportError:
         from ctypes import windll, WinError
 
         def setNoInherit(sock):
-            # mark the given socket fd as non-inheritable (Windows).
+            """Mark the given socket fd as non-inheritable to child processes"""
             if not windll.kernel32.SetHandleInformation(sock.fileno(), 1, 0):
                 raise WinError()
 
     except ImportError:
         # nothing available, define a dummy function
         def setNoInherit(sock):
+            """Mark the given socket fd as non-inheritable to child processes (dummy)"""
             pass
 
 
 class SocketConnection(object):
-    """A connection wrapper for sockets"""
+    """A wrapper class for plain sockets, containing various methods such as :meth:`send` and :meth:`recv`"""
     __slots__=["sock", "objectId"]
 
     def __init__(self, sock, objectId=None):
