@@ -10,6 +10,7 @@ import unittest
 import copy
 import logging
 import os, sys, time
+import warnings
 import Pyro4.configuration
 import Pyro4.core
 import Pyro4.errors
@@ -29,6 +30,8 @@ class Thing(object):
 
 
 class CoreTestsWithoutHmac(unittest.TestCase):
+    def setUp(self):
+        warnings.simplefilter("ignore")
     def testProxy(self):
         Pyro4.config.HMAC_KEY=None
         # check that proxy without hmac is possible
@@ -105,9 +108,6 @@ class CoreTests(unittest.TestCase):
         p=Pyro4.core.URI(uri)
         self.assertEqual(uri,str(p))
         self.assertEqual(uri,p.asString())
-        uri="PYRO:12345@./p:pipename"
-        p=Pyro4.core.URI(uri)
-        self.assertEqual(uri,str(p))
         uri="PYRO:12345@./u:sockname"
         p=Pyro4.core.URI(uri)
         self.assertEqual(uri,str(p))
@@ -123,7 +123,6 @@ class CoreTests(unittest.TestCase):
         self.assertEqual("PYRONAME",p.protocol)
         self.assertEqual("some_obj_name",p.object)
         self.assertEqual(None,p.host)
-        self.assertEqual(None,p.pipename)
         self.assertEqual(None,p.sockname)
         self.assertEqual(None,p.port)
         p=Pyro4.core.URI("PYRONAME:some_obj_name@host.com:9999")
@@ -136,15 +135,17 @@ class CoreTests(unittest.TestCase):
         self.assertEqual("PYRO",p.protocol)
         self.assertEqual("12345",p.object)
         self.assertEqual("host.com",p.host)
-        self.assertEqual(None,p.pipename)
         self.assertEqual(None,p.sockname)
         self.assertEqual(4444,p.port)
-        p=Pyro4.core.URI("PYRO:12345@./p:pipename")
-        self.assertEqual("12345",p.object)
-        self.assertEqual("pipename",p.pipename)
         p=Pyro4.core.URI("PYRO:12345@./u:sockname")
         self.assertEqual("12345",p.object)
         self.assertEqual("sockname",p.sockname)
+        p=Pyro4.core.URI("PYRO:12345@./u:/tmp/sockname")
+        self.assertEqual("12345",p.object)
+        self.assertEqual("/tmp/sockname",p.sockname)
+        p=Pyro4.core.URI("PYRO:12345@./u:../sockname")
+        self.assertEqual("12345",p.object)
+        self.assertEqual("../sockname",p.sockname)
 
     def testUriParsingPyroname(self):
         p=Pyro4.core.URI("PYRONAME:objectname")
@@ -185,9 +186,6 @@ class CoreTests(unittest.TestCase):
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRONAME:objname@nameserver:7766:bogus")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "FOOBAR:")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "FOOBAR:objid@hostname:7766")
-        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:12345@./p:pipename/slash")
-        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:12345@./u:sockname/slash")
-        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:12345@./p:pipename:9999")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:12345@./u:sockname:9999")
 
     def testUriUnicode(self):
@@ -198,7 +196,6 @@ class CoreTests(unittest.TestCase):
         self.assertTrue(type(p.protocol) is unicode)
         self.assertTrue(type(p.object) is unicode)
         self.assertTrue(type(p.host) is unicode)
-        self.assertEqual(None,p.pipename)
         self.assertEqual(None,p.sockname)
         self.assertEqual(4444,p.port)
 
@@ -247,10 +244,10 @@ class CoreTests(unittest.TestCase):
         self.assertEqual(p2,p3)
         
     def testLocation(self):
-        self.assertTrue(Pyro4.core.URI.isPipeOrUnixsockLocation("./p:name"))
-        self.assertTrue(Pyro4.core.URI.isPipeOrUnixsockLocation("./u:name"))
-        self.assertFalse(Pyro4.core.URI.isPipeOrUnixsockLocation("./x:name"))
-        self.assertFalse(Pyro4.core.URI.isPipeOrUnixsockLocation("foobar"))
+        self.assertTrue(Pyro4.core.URI.isUnixsockLocation("./u:name"))
+        self.assertFalse(Pyro4.core.URI.isUnixsockLocation("./p:name"))
+        self.assertFalse(Pyro4.core.URI.isUnixsockLocation("./x:name"))
+        self.assertFalse(Pyro4.core.URI.isUnixsockLocation("foobar"))
 
     def testMsgFactory(self):
         import hashlib, hmac
@@ -421,31 +418,49 @@ class CoreTests(unittest.TestCase):
 
 class RemoteMethodTests(unittest.TestCase):
     class BatchProxyMock(object):
+        def __copy__(self):
+            return self
+        def __enter__(self):
+            return self
+        def __exit__(self, *args):
+            pass
         def _pyroBatch(self):
-            return Pyro4.core._BatchProxy(self.pyroInvokeBatch)
-        def pyroInvokeBatch(self, calls, oneway=False):
+            return Pyro4.core._BatchProxyAdapter(self)
+        def _pyroInvokeBatch(self, calls, oneway=False):
             self.result=[]
-            for name, args, kwargs in calls:
-                if name=="error":
+            for methodname, args, kwargs in calls:
+                if methodname=="error":
                     self.result.append(Pyro4.core._ExceptionWrapper(ValueError("some exception")))
                     break  # stop processing the rest, this is what Pyro should do in case of an error in a batch
-                else:
-                    self.result.append("INVOKED %s args=%s kwargs=%s" % (name,args,kwargs))
+                elif methodname=="pause":
+                    time.sleep(args[0])
+                self.result.append("INVOKED %s args=%s kwargs=%s" % (methodname,args,kwargs))
             if oneway:
                 return
             else:
                 return self.result
 
     class AsyncProxyMock(object):
-        def _pyroAsync(self, callback=None):
-            return Pyro4.core._AsyncProxy(self.pyroInvoke, callback)
-        def pyroInvoke(self, methodname, vargs, kwargs, flags=0):
+        def __copy__(self):
+            return self
+        def __enter__(self):
+            return self
+        def __exit__(self, *args):
+            pass
+        def _pyroAsync(self):
+            return Pyro4.core._AsyncProxyAdapter(self)
+        def _pyroInvoke(self, methodname, vargs, kwargs, flags=0):
             if methodname=="pause_and_divide":
                 time.sleep(vargs[0])
                 return vargs[1]//vargs[2]
             else:
                 raise NotImplementedError(methodname)
 
+    def setUp(self):
+        Pyro4.config.HMAC_KEY=tobytes("testsuite")
+    def tearDown(self):
+        Pyro4.config.HMAC_KEY=None
+
     def testRemoteMethod(self):
         class ProxyMock(object):
             def invoke(self, name, args, kwargs):
@@ -492,6 +507,30 @@ class RemoteMethodTests(unittest.TestCase):
         results=batch(oneway=True)
         self.assertEqual(None, results)          # oneway always returns None
         self.assertEqual(4, len(proxy.result))   # should have done 4 calls, not 5
+        self.assertRaises(Pyro4.errors.PyroError, batch, oneway=True, async=True)   # oneway+async=booboo
+
+    def testBatchMethodAsync(self):
+        proxy=self.BatchProxyMock()
+        batch=Pyro4.batch(proxy)
+        self.assertEqual(None, batch.foo(42))
+        self.assertEqual(None, batch.bar("abc"))
+        self.assertEqual(None, batch.pause(0.5))    # pause shouldn't matter with async
+        self.assertEqual(None, batch.baz(42,"abc",arg=999))
+        begin=time.time()
+        asyncresult=batch(async=True)
+        duration=time.time()-begin
+        self.assertTrue(duration<0.1, "batch oneway with pause should still return almost immediately")
+        results=asyncresult.value
+        self.assertEqual(4, len(proxy.result))   # should have done 4 calls
+        result=next(results)
+        self.assertEqual("INVOKED foo args=(42,) kwargs={}",result)
+        result=next(results)
+        self.assertEqual("INVOKED bar args=('abc',) kwargs={}",result)
+        result=next(results)
+        self.assertEqual("INVOKED pause args=(0.5,) kwargs={}",result)
+        result=next(results)
+        self.assertEqual("INVOKED baz args=(42, 'abc') kwargs={'arg': 999}",result)
+        self.assertRaises(StopIteration, next, results)   # and now there should not be any more results
 
     def testBatchMethodReuse(self):
         proxy=self.BatchProxyMock()
@@ -520,14 +559,21 @@ class RemoteMethodTests(unittest.TestCase):
         self.assertTrue(result.ready())
 
     def testAsyncCallbackMethod(self):
-        def asyncCallback(value):
-            self.assertEqual(5,value)
-
+        class AsyncFunctionHolder(object):
+            asyncFunctionCount=0
+            def asyncFunction(self, value, amount=1):
+                self.asyncFunctionCount+=1
+                return value+amount
         proxy=self.AsyncProxyMock()
-        async=Pyro4.async(proxy, callback=asyncCallback)
+        async=Pyro4.async(proxy)
         result=async.pause_and_divide(1,10,2)  # returns immediately
-        self.assertFalse(result.ready())
-        # can't use result.value here because we're using a callback.
+        holder=AsyncFunctionHolder()
+        result.then(holder.asyncFunction, amount=2) \
+              .then(holder.asyncFunction, amount=4) \
+              .then(holder.asyncFunction)
+        value=result.value
+        self.assertEqual(10//2+2+4+1,value)
+        self.assertEqual(3,holder.asyncFunctionCount)
 
     def testAsyncMethodTimeout(self):
         proxy=self.AsyncProxyMock()
