@@ -3,7 +3,7 @@ Support code for the test suite.
 There's some Python 2.x <-> 3.x compatibility code here.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 import sys
